function [x_in, y_icdf_norm, y_icdf_mean, y_icdf_std, X, ref_pdf_norm, freq_y_icdf, psd_y_icdf_norm ]  = noise_generator(N, K_psd, typ_icdf, wariant_filtru, filename, do_draw)
 
if nargin == 0
  N = 1000
%   K_psd = 8192;
  K_psd = 512;
  typ_icdf = 'triangular' % typ docelowego rozkładu (pdf)
  wariant_filtru = 'FIR_D'
%   wariant_filtru = 'IIR_B'
  filename = 'test.mat'
end

if nargin < 6
  do_draw = true;
end

% # zapisanie zmiennej CQT do pliku w formacie matlaba
% from scipy.io import savemat as save
% save(filename, {"CQT": CQT}, do_compression=True)

% ============================================================================================ %
% zakładamy na wejściu rozkład gaussowski, tak że przejście przez filtr zachowa ten rozkład
% ============================================================================================ %
pd_gauss = makedist('Normal','mu',0,'sigma',1);
% 
% x_in = rand(1,N);
% x_in = pd_gauss.icdf(x_in); % konwersja szumu o rozkłądize równomiernym do szumu o zadanym rozkładzie (tutaj rozkład gaussowski)
x_in = randn(1,N); % bezpośrednia generacja szumu o rozkładize gaussowski

% %%Normalizacja szumu wejsciowego 
% x_normal = (x_in - mean(x_in)) / std(x_normal);     
% x_in = x_normal; 


% ============================================================================================ %
% zakładany rozkład wyjściowy
% ============================================================================================ %
switch typ_icdf
  case 'gaussowski'
    pd_icdf = makedist('Normal','mu',0,'sigma',1); 
    
  case 'sinus'     
    x = linspace(-3,3, 1001);
    Fx = cdf_(2,x);
    pd_icdf = makedist('PiecewiseLinear', 'x', x, 'Fx', Fx);
    
  case 'weibull'
     pd_icdf = makedist('Weibull','a',1,'b',1);
     
  case 'triangular'
     pd_icdf = makedist('Triangular','a',-1,'b',0,'c',1);
end

pd_uniform = makedist('Uniform');
% pd_normal = makedist('Normal');


% ============================================================================================ %
% zakładany filtr kształtujacy docelowe widmo
% ============================================================================================ %
if ~exist('wariant_filtru', 'var')
  wariant_filtru = 'off';
  wariant_filtru = 'IIR_B';
  wariant_filtru = 'FIR_C';
  warning('Nie ustawiono zmiennej wariant_filtru');
end
%Parameters of digital filter 
switch wariant_filtru
  case 'off' % wyłączony fitlr
    b = 1;
    a = 1;
  
  case 'FIR_A'
    b = 1;
    b(40 + 1) = -1/2;
    b(100 + 1) = 1/4;
    a = 1;

  case 'FIR_B'
    % prosty filtr uśredniający
    b = ones(1, 21)/21;
    a = 1;

  case 'FIR_C'
    % filtr pasmowo przepustowy
    b = firpm(105,[0 0.25 .3 .5 0.55 1],[0 0 1 1 0 0],'h');
    a = 1;

  case 'FIR_D'
    % filtr dolno pasmowy
    b = firpm(105,[0 0.25 .3 1],[1 1 0 0]);
    a = 1;

  case 'IIR_A'
    % filtr IIR rezonansowy z pojedynczym rezonansem
    b = 1;
    omega_1 = 0.1*(2*pi); % pulsacja rezonansu
    p = 0.95*[exp(j*omega_1), exp(-j*omega_1)]; % dwa bieguny sprzężone
    a = poly(p);

  case 'IIR_B'
    % filtr IIR rezonansowy z dwoma rezonansami
    b = 1;
    omega_1 = 0.1*(2*pi); % pulsacja rezonansu
    omega_2 = 0.25*(2*pi); % pulsacja drugiego rezonansu
    p = 0.95*[exp(j*omega_1), exp(j*omega_2), exp(-j*omega_1), exp(-j*omega_2)]; % 2 x dwa bieguny sprzężone
    a = poly(p);

  otherwise
    % współczynniki odpowiadające wyłączeniu filtru
    b = 1;
    a = 1;
    warning('nieobsługiwany wariant filtru');
end

% ============================================================
% Normalizacja współczynników filtru, tak żeby nie zmieniał on mocy szumu białego
K = 8192;
[H, f] = freqz(b, a, K, 1); % obliczanie charakterystyki częstotliwościowej filtru dla tych samych pulsacji, dla których obliczono psd
alfa = 1/sqrt(mean(abs(H).^2));
b = alfa * b;

[H, f] = freqz(b, a, K, 1); % obliczanie charakterystyki częstotliwościowej filtru dla tych samych pulsacji, dla których obliczono psd
% ============================================================

% ============================================================
% filtracja szumu gaussowskiego
% ============================================================
y = filter (b, a, x_in);

% ============================================================
% konwersja szumu o zadanym widmie i rozkładzie gaussowskim do szumu o rozkładzie równomiernym
% ============================================================
y_r = pd_gauss.cdf(y);

% ============================================================
% konwersja szumu o rozkładzie równomiernym do docelowego rozkładu
% ============================================================
y_icdf = pd_icdf.icdf(y_r);

% %Normalizacja szumu wynikowego
y_icdf_mean = mean(y_icdf)
y_icdf_std = std(y_icdf)
y_icdf_norm = (y_icdf - y_icdf_mean) / y_icdf_std;     

if do_draw == true
  figure(1)
  subplot(2,1,1)
  plot(x_in, 'b.')
  subplot(2,1,2)
  plot(y_icdf_norm, 'b.')
  % 1a. wykresy hist + założone wzorce
  figure(2)
  ha1 = subplot(4,2,1);
  h = histogram(ha1, x_in,101,'BinLimits',[-4,4],'Normalization', 'pdf');
  X = h.BinEdges(1:end-1)+diff(h.BinEdges)/2;
  ref_pdf = pd_gauss.pdf(X);
  title(ha1, 'histogram szumu wejsciowego')
  
  
  
  hold(ha1, 'on')
  plot(ha1, X, ref_pdf, 'r');
  hold(ha1, 'off')
  grid(ha1, 'on')
  legend(ha1, {'hist', 'pdf'})
  
  ha1b = subplot(4,2,2);
  h = histogram(ha1b, y,101,'BinLimits',[-4,4],'Normalization', 'pdf');
  X = h.BinEdges(1:end-1)+diff(h.BinEdges)/2;
  ref_pdf = pd_gauss.pdf(X);
  title(ha1b, 'histogram szumu po filtracji')
  
  hold(ha1b, 'on')
  plot(ha1b, X, ref_pdf, 'r');
  hold(ha1b, 'off')
  grid(ha1b, 'on')
  legend(ha1b, {'hist', 'pdf'})
  
  ha2a = subplot(4,2,3);
  h = histogram(ha2a, y_r,101,'BinLimits',[-4,4],'Normalization', 'pdf');
  X = h.BinEdges(1:end-1)+diff(h.BinEdges)/2;
  ref_pdf = pd_uniform.pdf(X);
  title(ha2a, 'histogram szumu przekonwertowanego do rozkładu równomiernego')
  
  hold(ha2a, 'on')
  plot(ha2a, X, ref_pdf, 'r');
  hold(ha2a, 'off')
  grid(ha2a, 'on')
  legend(ha2a, {'hist', 'pdf'})
  
  ha2 = subplot(4,2,4);
  h = histogram(ha2, y_icdf,101,'BinLimits',[-4,4],'Normalization', 'pdf');
  X = h.BinEdges(1:end-1)+diff(h.BinEdges)/2;
  hold(ha2, 'on')
  h_norm = histogram(ha2, y_icdf_norm,101,'BinLimits',[-4,4],'Normalization', 'pdf');
  hold(ha2, 'off')
  ref_pdf = pd_icdf.pdf(X);
  ref_pdf_norm = pd_icdf.pdf((X+y_icdf_mean)*y_icdf_std)*y_icdf_std;
  title(ha2, 'histogram szumu wyjsciowego')
end

[N_hist,EDGES] = histcounts(y_icdf,101,'BinLimits',[-4,4],'Normalization', 'pdf');
X = EDGES(1:end-1)+diff(EDGES)/2;
ref_pdf_edges = EDGES;
ref_pdf_norm = pd_icdf.pdf((X+y_icdf_mean)*y_icdf_std)*y_icdf_std;


if do_draw == true
  hold(ha2, 'on')
  plot(ha2, X, ref_pdf, 'r');
  plot(ha2, X, ref_pdf_norm, 'm');
  hold(ha2, 'off')
  grid(ha2, 'on')
  legend(ha2, {'hist', 'hist norm', 'pdf', 'pdf norm'})
end

% 1b. wykresy widma + założone wzorce
freq_lim = [0,pi]/(2*pi);
if length(x_in) >= K_psd/2
    window = rectwin(K_psd/2);
else
    window = rectwin(length(x_in));
end

[psd_x,freq_x] = pwelch(x_in, window, 0, K_psd, 1);
psd_x = psd_x/2; % korekta normalisacji widma psd % dla sygnału rzeczywistego
[psd_y,freq_y] = pwelch(y, window, 0, K_psd, 1);
psd_y = psd_y/2; % korekta normalisacji widma psd % ???
[psd_y_r,freq_y_r] = pwelch(y_r-(0.5), window, 0, K_psd, 1);
psd_y_r = psd_y_r/2; % korekta normalisacji widma psd % ???
[psd_y_icdf_norm,freq_y_icdf] = pwelch(y_icdf_norm, window, 0, K_psd, 1);
psd_y_icdf_norm = psd_y_icdf_norm/2; % korekta normalisacji widma psd % ???


if do_draw == true
  ha3 = subplot(2,1,2); 
  plot(ha3,freq_x, psd_x, 'b');
  legend_strings = { 'psd(x) - widmo szumu wejsciowego' };
  grid(ha3, 'on')
  xlim(ha3 ,freq_lim)
  
  hold on 
  plot(ha3,freq_y, psd_y, 'r');
  legend_strings{end+1} = 'psd(y) - widmo szumu wejściowego po filtracji';
  
  
  plot(ha3,freq_y_r, psd_y_r, 'm--');
  legend_strings{end+1} = 'psd(y_r) - widmo szumu po konwersji do rozkładu równomiernego';
  
  plot(ha3,freq_y, psd_y_icdf_norm, 'k');
  legend_strings{end+1} = 'psd\_y - widmo szumu wyjściowego';
  
  set(plot(f, abs(H).^2, 'k--'), 'LineWidth', 2)
  legend_strings{end+1} = 'abs(H)^2';
  hold off
  legend(legend_strings)
  
  
  fig = figure(3),
  h_ref = histogram(y_icdf_norm,101,'BinLimits',[-4,4]','Normalization','probability');
  xdata = h_ref.Values
  ydata = h_ref.BinEdges
end

[xdata, ydata]  = histcounts(y_icdf_norm,101,'BinLimits',[-4,4]','Normalization','probability');

% 2. generacja odpowiedzi impulsowej na bazie wzorca psd

% 2. generacja odpowiedzi impulsowej na bazie wzorca psd
[H, f] = freqz(b, a, K_psd, 'whole', 1); % obliczanie charakterystyki częstotliwościowej filtru dla tych samych pulsacji, dla których obliczono psd
psd_ref = abs(H).^2;
psd_ref = psd_ref(1:(K_psd/2+1)); % bieżemy tylko dodatnie częstotliwości

save(filename, 'N', 'x_in', 'K_psd', 'typ_icdf', 'wariant_filtru', 'y_icdf_norm', 'y_icdf_mean', 'y_icdf_std', 'X', 'ref_pdf_edges', 'ref_pdf_norm', 'freq_y_icdf', 'psd_ref', 'psd_y_icdf_norm', 'f', 'H', 'xdata', 'ydata');

