#wczytanie modelu
import os
import shutil
import numpy as np
import scipy.io
from keras.saving.model_config import model_from_json
from scipy import signal
import matplotlib.pyplot as plt

os.environ["KMP_DUPLICATE_LIB_OK"]="TRUE"

nosignals = 20
NN_in_out_size = 500
K_psd = 256
    
# # model_time_str = '2022_09_17-21_38_34' # nosignals = 10
# # model_time_str = '2022_09_17-21_51_42' # nosignals = 250 
model_time_str = '2022_09_17-23_06_10'
model_train_ind = 500 # 500 # 250 # 100 # 10

json_file = open(f"model-{model_time_str}-ind_{model_train_ind}.json", 'r')
loaded_model_json = json_file.read()
json_file.close()
loaded_model = model_from_json(loaded_model_json)
# load weights into new model
loaded_model.load_weights(f"model-{model_time_str}-ind_{model_train_ind}.h5")
print("Loaded model from disk")

# tutaj wyciągnięcie tych sygnałów z pliku matlabowego

def get_samples_from_file(filename, N_seg=20000):
    # filename - nazwa pliku z datasetem
    # N_seg - długość segmentów, na które podzielony ma być dataset
    mat = scipy.io.loadmat(filename)

    N_all = mat['N'] # liczba próbek sygnału w dataset



    no_of_segments = int(np.floor(N_all / N_seg)) # liczba segmentów, na które można podzielidź dataset
    Samples_in = []
    n_seg = np.arange(N_seg)
    for j in range(no_of_segments):
        # wycinamy kolejne segmenty
        inner_list = mat['x_in'][0,j*N_seg + n_seg]
        Samples_in.append(inner_list)

    return Samples_in, no_of_segments



filename = f'./data/test_0.mat'
mat = scipy.io.loadmat(filename)
xdata_mat = mat['ydata']
ydata_mat = mat['xdata']
xdata = xdata_mat.flatten()
ydata = ydata_mat.flatten()

granted_pdf_edges = mat['ref_pdf_edges'].flatten()
granted_pdf = mat['ref_pdf_norm'].flatten()
granted_psd = mat['psd_ref'].flatten()


parent_dir = os.getcwd()
new_dir = "Predictions"

path = os.path.join(parent_dir,new_dir)

try:
    print("Creating...." + path)
    os.makedirs(path)
except:
    print("folder "+ path +" already exist")
#     print("deleting current "+ path +" folder")
#     shutil.rmtree(path)
#     print("Creating new...." + path)
#     os.makedirs(path)

figname = "\Prediction-" + model_time_str + f"-ind_{model_train_ind}" # + str(ind) + "_" + str(i)
save_path = path + figname
print(save_path)
fig = plt.figure(figname)
axs = fig.subplots(3)
counter = 1
for ind in range(nosignals):
    filename = f'./data/test_{ind}.mat'
    Samples_in, no_of_segments = get_samples_from_file(filename, N_seg=NN_in_out_size)
    Xpred = np.asarray(Samples_in).astype(np.float32)
    prediction = loaded_model.predict(Xpred)
    print(len(prediction))

    for i in range(no_of_segments):
        f2, psd = signal.welch(prediction[i, :], window=signal.get_window('boxcar', int(K_psd/2)), nfft=K_psd, scaling='density')
        psd = 0.5 * psd # correct vs Matlab pwelch
        if (ind == 0) & (i == 0):
            f_all = f2
            psd_all = psd

            # jako przykład
            axs[0].plot(prediction[i, :])
            axs[1].plot(f2, psd)
        else:
            psd_all = psd_all + psd

        hist1 = np.histogram(prediction[i, :], bins=granted_pdf_edges, density = True)
        # hist1 = np.histogram(prediction[i, 100:], bins=granted_pdf_edges, density = True)
                             
        if (ind == 0) & (i == 0):
            hist_all = hist1[0]

            # jako przykład
            hist1 = axs[2].hist(prediction[i, :], bins=granted_pdf_edges,
                                density=True,
                                histtype='bar', alpha=0.75)
        else:
            hist_all = hist_all + hist1[0]

        print("Generating " + figname + "...")
        print( str(nosignals*no_of_segments - counter) +  " left")
        counter = counter+1

psd_all = psd_all / (nosignals * no_of_segments)
hist_all = hist_all / (nosignals * no_of_segments)

granted_pdf_centers = granted_pdf_edges[0:-1]+(granted_pdf_edges[1]-granted_pdf_edges[0])/2
axs[1].plot(f_all, psd_all)
axs[1].plot(f_all, granted_psd)
axs[2].plot(granted_pdf_centers, hist_all) 
axs[2].plot(granted_pdf_centers, granted_pdf)

fig.savefig(save_path)
plt.close(fig)

